# Generated by Django 4.1.4 on 2023-11-20 03:51

from django.db import migrations, models
import django.db.models.deletion
import rest.models.base
import rest.models.metadata


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0016_authsession_buid'),
        ('incident', '0010_incident_category_incident_component_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='Ticket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('category', models.CharField(db_index=True, max_length=200)),
                ('priority', models.IntegerField(db_index=True, default=10)),
                ('title', models.CharField(max_length=200)),
                ('description', models.TextField()),
                ('status', models.CharField(db_index=True, default=None, max_length=32, null=True)),
                ('state', models.IntegerField(default=0)),
                ('component', models.CharField(db_index=True, default=None, max_length=200, null=True)),
                ('component_id', models.IntegerField(blank=True, db_index=True, default=None, null=True)),
                ('assigned_to', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tickets', to='account.member')),
                ('created_by', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='account.member')),
                ('group', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tickets', to='account.group')),
                ('incident', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tickets', to='incident.incident')),
            ],
            bases=(models.Model, rest.models.base.RestModel, rest.models.metadata.MetaDataModel),
        ),
    ]
