# Generated by Django 4.1.1 on 2022-09-24 21:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0009_alter_member_phone_number'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AuthToken',
        ),
    ]
