# Generated by Django 4.1.4 on 2023-05-04 20:11

from django.db import migrations, models
import rest.models


class Migration(migrations.Migration):

    dependencies = [
        ('metrics', '0003_metrics_expires'),
    ]

    operations = [
        migrations.CreateModel(
            name='EODMetrics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('end', models.DateTimeField(db_index=True)),
                ('uuid', models.SlugField(max_length=250, unique=True)),
                ('slug', models.CharField(db_index=True, default=None, max_length=200, null=True)),
                ('expires', models.DateTimeField(db_index=True, default=None, null=True)),
                ('granularity', models.CharField(db_index=True, max_length=64)),
                ('component', models.CharField(db_index=True, default=None, max_length=255, null=True)),
                ('component_pk', models.IntegerField(db_index=True, default=None, null=True)),
                ('group_id', models.IntegerField(db_index=True, default=None, null=True)),
                ('k1', models.SlugField(default=None, max_length=64, null=True)),
                ('v1', models.IntegerField(db_index=True, default=0)),
                ('k2', models.SlugField(default=None, max_length=64, null=True)),
                ('v2', models.IntegerField(db_index=True, default=0)),
                ('k3', models.SlugField(default=None, max_length=64, null=True)),
                ('v3', models.IntegerField(default=0)),
                ('k4', models.SlugField(default=None, max_length=64, null=True)),
                ('v4', models.IntegerField(default=0)),
                ('k5', models.SlugField(default=None, max_length=64, null=True)),
                ('v5', models.IntegerField(default=0)),
                ('k6', models.SlugField(default=None, max_length=64, null=True)),
                ('v6', models.IntegerField(default=0)),
                ('k7', models.SlugField(default=None, max_length=64, null=True)),
                ('v7', models.IntegerField(default=0)),
                ('k8', models.SlugField(default=None, max_length=64, null=True)),
                ('v8', models.IntegerField(default=0)),
                ('k9', models.SlugField(default=None, max_length=64, null=True)),
                ('v9', models.IntegerField(default=0)),
                ('k10', models.SlugField(default=None, max_length=64, null=True)),
                ('v10', models.IntegerField(default=0)),
                ('k11', models.SlugField(default=None, max_length=64, null=True)),
                ('v11', models.IntegerField(default=0)),
                ('k12', models.SlugField(default=None, max_length=64, null=True)),
                ('v12', models.IntegerField(default=0)),
                ('k13', models.SlugField(default=None, max_length=64, null=True)),
                ('v13', models.IntegerField(default=0)),
                ('k14', models.SlugField(default=None, max_length=64, null=True)),
                ('v14', models.IntegerField(default=0)),
            ],
            bases=(models.Model, rest.models.RestModel),
        ),
    ]
