# Generated by Django 4.0.2 on 2022-02-26 15:59

from django.db import migrations, models
import rest.fields


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0001_initial'),
        ('medialib', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='member',
            name='phone_number',
            field=rest.fields.FormattedField(default=None, max_length=64, null=True),
        ),

        migrations.AddField(
            model_name='member',
            name='picture',
            field=models.ForeignKey(blank=True, help_text='Profile picture', null=True, on_delete=models.deletion.CASCADE, related_name='+', to='medialib.mediaitem'),
        )
    ]