# Generated by Django 4.1.4 on 2023-11-15 19:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0014_alter_notificationmemberrecord_member'),
    ]

    operations = [
        migrations.AddField(
            model_name='memberdevice',
            name='buid',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=128, null=True),
        ),
    ]
